@@ -1,10 +1,12 @@
+import os
 import yafowil.utils
 
+
 def register():
     import yafowil.common
     import yafowil.compound
     import yafowil.table
-    import yafowil.plans
+
 
 for ep in yafowil.utils.get_entry_points('register'):
-    ep.load()()
\ No newline at end of file
+    ep.load()()
