@@ -12,29 +12,22 @@ def get_entry_points(ns=None):
 
 
 def get_plugin_names(ns=None):
-    return list(set([_.module_name for _ in get_entry_points(ns=ns)]))
+    return list(set([_.dist.project_name for _ in get_entry_points(ns=ns)]))
 
 
-def get_resource_directory(module_name):
-    for ep in get_entry_points(ns='resourcedir'):
-        if ep.module_name != module_name:
+def get_example(example_name):
+    for ep in get_entry_points(ns='example'):
+        if ep.dist.project_name != example_name:
             continue
-        return ep.load()()
+        info = ep.load()()
+        return info
 
 
-def _get_filepaths(module_name, ns, thirdparty):
-    for ep in get_entry_points(ns=ns):
-        if ep.module_name == module_name:
-            return ep.load()(thirdparty=thirdparty)
-    return []
-
-
-def get_javascripts(module_name, thirdparty=True):
-    return _get_filepaths(module_name, 'javascripts', thirdparty)
-
-
-def get_stylesheets(module_name, thirdparty=True):
-    return _get_filepaths(module_name, 'stylesheets', thirdparty)
+def get_example_names():
+    result = []
+    for ep in get_entry_points(ns='example'):
+        result.append(ep.dist.project_name)
+    return result
 
 
 class Unset(object):
@@ -93,23 +86,23 @@ class Tag(object):
 
     def __call__(self, tag_name, *inners, **attributes):
         """Generates some xml/html tag.
-            
+
         ``tagname``
             name of a valid tag
-            
+
         ``inners``
             inner content of the tag. If empty a closed tag is generated
-        
+
         ``attributes``
-            attributes of the tag, leading or trailing ``_`` underscores are 
+            attributes of the tag, leading or trailing ``_`` underscores are
             omitted from keywords.
-    
+
         Example::
-    
-            >>> tag('p', 'Lorem Ipsum.', u'Hello World!', 
+
+            >>> tag('p', 'Lorem Ipsum.', u'Hello World!',
             ...     class_='fancy', id='2f5b8a234ff')
             <p class="fancy" id="2f5b8a234ff">Lorem Ipsum. Hello World.</p>
-        
+
         """
         cl = list()
         for key, value in attributes.items():
@@ -121,7 +114,8 @@ class Tag(object):
             cl.append((key.strip('_'), value))
         attributes = u''
         if cl:
-            attributes = u' %s' % u' '.join(sorted([u'%s="%s"' % _ for _ in cl]))
+            attributes = u' %s' % \
+                         u' '.join(sorted([u'%s="%s"' % _ for _ in cl]))
         cl = list()
         for inner in inners:
             inner = self.translate(inner)
@@ -161,15 +155,16 @@ class managedprops(object):
 
 def cssid(widget, prefix, postfix=None):
     path = widget.dottedpath.replace('.', '-')
-    id = "%s-%s" % (prefix, path)
+    cssid = "%s-%s" % (prefix, path)
     if postfix is not None:
-        id = '%s-%s' % (id, postfix)
-    return id
+        cssid = '%s-%s' % (cssid, postfix)
+    return cssid
 
 
-css_managed_props = ['class', 'error_class', 'error_class_default',
+css_managed_props = ['class', 'class_add', 'error_class', 'error_class_default',
                      'required_class', 'required_class_default']
 
+
 def cssclasses(widget, data, classattr='class', additional=[]):
     _classes = list()
     attrs = widget.attrs
@@ -184,6 +179,14 @@ def cssclasses(widget, data, classattr='class', additional=[]):
         else:
             _classes.append(attrs['required_class_default'])
     if attrs[classattr]:
-        _classes += attrs[classattr].split()
+        if callable(attrs[classattr]):
+            _classes += attrs[classattr](widget, data).split()
+        else:
+            _classes += attrs[classattr].split()
+    if attrs['class_add']:
+        if callable(attrs['class_add']):
+            _classes += attrs['class_add'](widget, data).split()
+        else:
+            _classes += attrs['class_add'].split()
     _classes += additional
     return _classes and ' '.join(sorted(_classes)) or None
